﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace FindSmallest
{
    public class Program
    {
        private static readonly int[][] Data =
        {
            new[]{1,5,4,2}, 
            new[]{3,2,4,11,4},
            new[] {33,2,3,-1, 10},
            new[] {3,2,8,9,-1},
            new[] {1, 22,1,9,-3, 5}
        };

        static void Main()
        {
            //SequentialTheOriginal();
            ParallelPool();
        }

        public static int FindSmallest(IList<int> numbers)
        {
            if (numbers.Count < 1)
            {
                throw new ArgumentException("There must be at least one element in the array");
            }

            int smallestSoFar = numbers[0];
            foreach (int number in numbers)
            {
                if (number < smallestSoFar)
                {
                    smallestSoFar = number;
                }
            }
            return smallestSoFar;
        }

        // More general than FindSmallest.
        // Finds the most "interesting" element using the comparision
        // This was an extra in the exercise text.
        public static int FindElement(IList<int> numbers, Comparison<int> comparison)
        {
            if (numbers.Count < 1)
            {
                throw new ArgumentException("There must be at least one element in the array");
            }
            int elementSoFar = numbers[0];
            foreach (int number in numbers)
            {
                int x = number;
                if (comparison(x, elementSoFar) > 0)
                {
                    elementSoFar = number;
                }
            }
            return elementSoFar;
        }


        /// <summary>
        /// Sequential (not concurrent)
        /// As in the original exercise text
        /// </summary>
        private static void SequentialTheOriginal()
        {
            foreach (int[] data in Data)
            {
                int smallest = FindSmallest(data);
                Console.WriteLine(String.Join(", ", data) + ": " + smallest);
            }
        }

        private static void ParallelPool()
        {
            Task<int>[] tasks = new Task<int>[Data.Length];
            int index = 0;
            foreach (int[] data in Data)
            {
                int[] dataCopy = data;
                Task<int> task = Task.Run(() => FindSmallest(dataCopy));
                //Task<int> task = Task.Run(() => FindElement(dataCopy, Largest));
                tasks[index++] = task;
            }
            for (int i = 0; i < Data.Length; i++)
            {
                int result = tasks[i].Result;
                Console.WriteLine(String.Join("; ", Data[i]) + ": " + result);
            }
        }

        public static int Lowest(int a, int b)
        {
            return b - a;
        }

        public static int Highest(int a, int b)
        {
            return a - b;
        }
    }
}